package coms6998_11.cs.columbia.edu.calculator;

import android.os.Bundle;
import android.app.Activity;
import android.view.Menu;
import android.view.View;
import android.widget.TextView;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;

import com.google.android.gcm.GCMBroadcastReceiver;
import com.google.android.gcm.GCMRegistrar;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.widget.TextView;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;

public class CalculatorActivity extends Activity {
	private InputBuffer buffer;
	private CalculatorBrain brain;
    private boolean inMiddleEnterNum;
	private String error;
	private int screenlines;
	
	 private GCMBroadcastReceiver mGCMReceiver;
	 private IntentFilter mOnRegisteredFilter;

	 private TextView mStatus;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        this.buffer = new InputBuffer();
        this.brain = new CalculatorBrain();
        this.inMiddleEnterNum = true;
       
        this.screenlines = 2;
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_calculator);
        mStatus = (TextView) findViewById(R.id.status);

        mGCMReceiver = new GCMBroadcastReceiver();
        mOnRegisteredFilter = new IntentFilter();
        mOnRegisteredFilter.addAction(Constants.ACTION_ON_REGISTERED);

        if (Constants.SENDER_ID == null) {
          mStatus.setText("Missing SENDER_ID");
          return;
        }
        if (Constants.SERVER_URL == null) {
          mStatus.setText("Missing SERVER_URL");
          return;
        }

        GCMRegistrar.checkDevice(this);
        GCMRegistrar.checkManifest(this);
        final String regId = GCMRegistrar.getRegistrationId(this);
        if (!regId.equals("")) {
          sendIdToServer(regId);
        } else {
          GCMRegistrar.register(this, Constants.SENDER_ID);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_calculator, menu);
        return true;
    }

    public void updateDisplay() {
        final TextView display = (TextView) findViewById(R.id.display);
        StringBuilder text;
        if (this.buffer.isEmpty() && this.error == null) {
          if (this.brain.isEmpty()) {
            // Display zero rather than a totally empty display
            text = new StringBuilder();
            for (int i = 1; i < this.screenlines - 1; i++) {
              text.append('\n');
            }
            text.append('0');
            final int scale = this.brain.getScale();
            if (scale > 0) {
              text.append('.');
              for (int i = 0; i < scale; i++) {
                text.append('0');
              }
            }
          } else {
            text = this.brain.toString(this.screenlines);
          }
        } else {
          text = this.brain.toString(this.screenlines - 1);
          text.append("\n");
          if (this.error == null) {
            text.append(this.buffer.get());
          } else {
            text.append(this.error);
            this.error = null;
          }
        }
        display.setText(text);
      }
    public void digitClickHandler(final View v) {	   
 	   final String key = (String) v.getTag();
 	   final char c = key.charAt(0);
       this.buffer.append(c);
       this.inMiddleEnterNum = true;
       this.updateDisplay();
    }
    
   public void operatorClickHandler(final View v) {	   
	   final String key = (String) v.getTag();
	   final char c = key.charAt(0);
	   if(inMiddleEnterNum) {
		   enterClickHandler(v);
	   } 
	   this.brain.performOperation(c);
	   this.updateDisplay();
   }
   
   public void pushOperand() {
	      if (!this.buffer.isEmpty()) {
	        final String num = this.buffer.get();
	        this.brain.push(num);
	        this.buffer.zap();
	      }
   }
   
    public void enterClickHandler(final View v) { 
    	pushOperand();
    	inMiddleEnterNum = false;
    }
    
    public void clearClickHandler(final View v) {  
        this.brain.emptyStack();
        
        final TextView display = (TextView) findViewById(R.id.display);
        display.setText("0");
    }
    private void sendIdToServer(String regId) {
        String status = getString(R.string.gcm_registration, regId);
        mStatus.setText(status);
        (new SendRegistrationIdTask(regId)).execute();
      }

      @Override
      public void onResume() {
        super.onResume();
        registerReceiver(mGCMReceiver, mOnRegisteredFilter);
      }

      @Override
      public void onPause() {
        super.onPause();
        unregisterReceiver(mGCMReceiver);
      }

      private class GCMReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
          String regId = intent.getStringExtra(Constants.FIELD_REGISTRATION_ID);
          sendIdToServer(regId);
        }
      }

      private final class SendRegistrationIdTask extends
          AsyncTask<String, Void, HttpResponse> {
        private String mRegId;

        public SendRegistrationIdTask(String regId) {
          mRegId = regId;
        }

        @Override
        protected HttpResponse doInBackground(String... regIds) {
          String url = Constants.SERVER_URL + "/register";
          HttpPost httppost = new HttpPost(url);

          try {
            List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>(1);
            nameValuePairs.add(new BasicNameValuePair("regId", mRegId)); //reg_id
            httppost.setEntity(new UrlEncodedFormEntity(nameValuePairs));
            HttpClient httpclient = new DefaultHttpClient();
            return httpclient.execute(httppost);
          } catch (ClientProtocolException e) {
            Log.e(Constants.TAG, e.getMessage(), e);
          } catch (IOException e) {
            Log.e(Constants.TAG, e.getMessage(), e);
          }

          return null;
        }

        @Override
        protected void onPostExecute(HttpResponse response) {
          if (response == null) {
            Log.e(Constants.TAG, "HttpResponse is null");
            return;
          }

          StatusLine httpStatus = response.getStatusLine();
          if (httpStatus.getStatusCode() != 200) {
            Log.e(Constants.TAG, "Status: " + httpStatus.getStatusCode());
            mStatus.setText(httpStatus.getReasonPhrase());
            return;
          }

          String status = getString(R.string.server_registration, mRegId);
          mStatus.setText(status);
        }
      }
}
